
from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x57Dec9cC7f492d6583c773e2E7ad66dcDc6940Fb",
    "sd": "0xad2ea7b7e49be15918E4917736E86ff7feA57a6A"
}

ABI = """[
    {"type": "constructor", "inputs": [{"name": "_govToken", "type": "address", "internalType": "contract ICrystalToken"}, {"name": "_devaddr", "type": "address", "internalType": "address"}, {"name": "_liquidityaddr", "type": "address", "internalType": "address"}, {"name": "_comfundaddr", "type": "address", "internalType": "address"}, {"name": "_founderaddr", "type": "address", "internalType": "address"}, {"name": "_rewardPerSecond", "type": "uint256", "internalType": "uint256"}, {"name": "_startTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "_halvingAfterTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "_rewardMultiplier", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_timeDeltaStartStage", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_timeDeltaEndStage", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_userFeeStage", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_devFeeStage", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "nonpayable"},
    {"name": "Deposit", "type": "event", "inputs": [{"name": "user", "type": "address", "indexed": true, "internalType": "address"}, {"name": "pid", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "EmergencyWithdraw", "type": "event", "inputs": [{"name": "user", "type": "address", "indexed": true, "internalType": "address"}, {"name": "pid", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "OwnershipTransferred", "type": "event", "inputs": [{"name": "previousOwner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "newOwner", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "SendGovernanceTokenReward", "type": "event", "inputs": [{"name": "user", "type": "address", "indexed": true, "internalType": "address"}, {"name": "pid", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "lockAmount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Withdraw", "type": "event", "inputs": [{"name": "user", "type": "address", "indexed": true, "internalType": "address"}, {"name": "pid", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "FINISH_BONUS_AT_TIMESTAMP", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "HALVING_AT_TIMESTAMP", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "PERCENT_FOR_COM", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "PERCENT_FOR_DEV", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "PERCENT_FOR_FOUNDERS", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "PERCENT_FOR_LP", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "PERCENT_LOCK_BONUS_REWARD", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "REWARD_MULTIPLIER", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "REWARD_PER_SECOND", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "START_TIMESTAMP", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "add", "type": "function", "inputs": [{"name": "_allocPoint", "type": "uint256", "internalType": "uint256"}, {"name": "_lpToken", "type": "address", "internalType": "contract IERC20"}, {"name": "_withUpdate", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "addAuthorized", "type": "function", "inputs": [{"name": "_toAdd", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "authorized", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "bonusFinishUpdate", "type": "function", "inputs": [{"name": "_newFinish", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "claimReward", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "claimRewards", "type": "function", "inputs": [{"name": "_pids", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "comUpdate", "type": "function", "inputs": [{"name": "_newCom", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "comfundaddr", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "deposit", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "dev", "type": "function", "inputs": [{"name": "_devaddr", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "devFeeStage", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "devaddr", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "emergencyWithdraw", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "founderUpdate", "type": "function", "inputs": [{"name": "_newFounder", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "founderaddr", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "getLockPercentage", "type": "function", "inputs": [{"name": "_from", "type": "uint256", "internalType": "uint256"}, {"name": "_to", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getMultiplier", "type": "function", "inputs": [{"name": "_from", "type": "uint256", "internalType": "uint256"}, {"name": "_to", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getNewRewardPerSecond", "type": "function", "inputs": [{"name": "pid1", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getPoolReward", "type": "function", "inputs": [{"name": "_from", "type": "uint256", "internalType": "uint256"}, {"name": "_to", "type": "uint256", "internalType": "uint256"}, {"name": "_allocPoint", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "forDev", "type": "uint256", "internalType": "uint256"}, {"name": "forFarmer", "type": "uint256", "internalType": "uint256"}, {"name": "forLP", "type": "uint256", "internalType": "uint256"}, {"name": "forCom", "type": "uint256", "internalType": "uint256"}, {"name": "forFounders", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "govToken", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract ICrystalToken"}], "stateMutability": "view"},
    {"name": "halvingUpdate", "type": "function", "inputs": [{"name": "_newHalving", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "liquidityaddr", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "lockUpdate", "type": "function", "inputs": [{"name": "_newlock", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "lockcomUpdate", "type": "function", "inputs": [{"name": "_newcomlock", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "lockdevUpdate", "type": "function", "inputs": [{"name": "_newdevlock", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "lockfounderUpdate", "type": "function", "inputs": [{"name": "_newfounderlock", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "locklpUpdate", "type": "function", "inputs": [{"name": "_newlplock", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "lpUpdate", "type": "function", "inputs": [{"name": "_newLP", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "massUpdatePools", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "owner", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "pendingReward", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}, {"name": "_user", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "poolExistence", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "contract IERC20"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "poolId1", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "poolInfo", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "lpToken", "type": "address", "internalType": "contract IERC20"}, {"name": "allocPoint", "type": "uint256", "internalType": "uint256"}, {"name": "lastRewardTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "accGovTokenPerShare", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "poolLength", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "reclaimTokenOwnership", "type": "function", "inputs": [{"name": "_newOwner", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "removeAuthorized", "type": "function", "inputs": [{"name": "_toRemove", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "renounceOwnership", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "rewardMulUpdate", "type": "function", "inputs": [{"name": "_newMulReward", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "rewardUpdate", "type": "function", "inputs": [{"name": "_newReward", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "set", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}, {"name": "_allocPoint", "type": "uint256", "internalType": "uint256"}, {"name": "_withUpdate", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setDevFeeStage", "type": "function", "inputs": [{"name": "_devFees", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStageEnds", "type": "function", "inputs": [{"name": "_timeEnds", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStageStarts", "type": "function", "inputs": [{"name": "_timeStarts", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setUserFeeStage", "type": "function", "inputs": [{"name": "_userFees", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startTimestampUpdate", "type": "function", "inputs": [{"name": "_newstarttimestamp", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "timeDeltaEndStage", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "timeDeltaStartStage", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "totalAllocPoint", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "transferOwnership", "type": "function", "inputs": [{"name": "newOwner", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "updatePool", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "usdOracle", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "userDelta", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "userFeeStage", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "userInfo", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "amount", "type": "uint256", "internalType": "uint256"}, {"name": "rewardDebt", "type": "uint256", "internalType": "uint256"}, {"name": "rewardDebtAtTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "lastWithdrawTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "firstDepositTimestamp", "type": "uint256", "internalType": "uint256"}, {"name": "timeDelta", "type": "uint256", "internalType": "uint256"}, {"name": "lastDepositTimestamp", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "withdraw", "type": "function", "inputs": [{"name": "_pid", "type": "uint256", "internalType": "uint256"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class MasterGardener(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def finish_bonus_at_timestamp(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.FINISH_BONUS_AT_TIMESTAMP().call(block_identifier=block_identifier)

    def halving_at_timestamp(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.HALVING_AT_TIMESTAMP(a).call(block_identifier=block_identifier)

    def percent_for_com(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.PERCENT_FOR_COM().call(block_identifier=block_identifier)

    def percent_for_dev(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.PERCENT_FOR_DEV().call(block_identifier=block_identifier)

    def percent_for_founders(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.PERCENT_FOR_FOUNDERS().call(block_identifier=block_identifier)

    def percent_for_lp(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.PERCENT_FOR_LP().call(block_identifier=block_identifier)

    def percent_lock_bonus_reward(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.PERCENT_LOCK_BONUS_REWARD(a).call(block_identifier=block_identifier)

    def reward_multiplier(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.REWARD_MULTIPLIER(a).call(block_identifier=block_identifier)

    def reward_per_second(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.REWARD_PER_SECOND().call(block_identifier=block_identifier)

    def start_timestamp(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.START_TIMESTAMP().call(block_identifier=block_identifier)

    def add(self, cred:Credentials, _alloc_point:uint256, _lp_token:address, _with_update:bool) -> TxReceipt:
        tx = self.contract.functions.add(_alloc_point, _lp_token, _with_update)
        return self.send_transaction(tx, cred)

    def add_authorized(self, cred:Credentials, _to_add:address) -> TxReceipt:
        tx = self.contract.functions.addAuthorized(_to_add)
        return self.send_transaction(tx, cred)

    def authorized(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.authorized(a).call(block_identifier=block_identifier)

    def bonus_finish_update(self, cred:Credentials, _new_finish:uint256) -> TxReceipt:
        tx = self.contract.functions.bonusFinishUpdate(_new_finish)
        return self.send_transaction(tx, cred)

    def claim_reward(self, cred:Credentials, _pid:uint256) -> TxReceipt:
        tx = self.contract.functions.claimReward(_pid)
        return self.send_transaction(tx, cred)

    def claim_rewards(self, cred:Credentials, _pids:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.claimRewards(_pids)
        return self.send_transaction(tx, cred)

    def com_update(self, cred:Credentials, _new_com:address) -> TxReceipt:
        tx = self.contract.functions.comUpdate(_new_com)
        return self.send_transaction(tx, cred)

    def comfundaddr(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.comfundaddr().call(block_identifier=block_identifier)

    def deposit(self, cred:Credentials, _pid:uint256, _amount:uint256) -> TxReceipt:
        tx = self.contract.functions.deposit(_pid, _amount)
        return self.send_transaction(tx, cred)

    def dev(self, cred:Credentials, _devaddr:address) -> TxReceipt:
        tx = self.contract.functions.dev(_devaddr)
        return self.send_transaction(tx, cred)

    def dev_fee_stage(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.devFeeStage(a).call(block_identifier=block_identifier)

    def devaddr(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.devaddr().call(block_identifier=block_identifier)

    def emergency_withdraw(self, cred:Credentials, _pid:uint256) -> TxReceipt:
        tx = self.contract.functions.emergencyWithdraw(_pid)
        return self.send_transaction(tx, cred)

    def founder_update(self, cred:Credentials, _new_founder:address) -> TxReceipt:
        tx = self.contract.functions.founderUpdate(_new_founder)
        return self.send_transaction(tx, cred)

    def founderaddr(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.founderaddr().call(block_identifier=block_identifier)

    def get_lock_percentage(self, _from:uint256, _to:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getLockPercentage(_from, _to).call(block_identifier=block_identifier)

    def get_multiplier(self, _from:uint256, _to:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getMultiplier(_from, _to).call(block_identifier=block_identifier)

    def get_new_reward_per_second(self, pid1:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getNewRewardPerSecond(pid1).call(block_identifier=block_identifier)

    def get_pool_reward(self, _from:uint256, _to:uint256, _alloc_point:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint256, uint256, uint256]:
        return self.contract.functions.getPoolReward(_from, _to, _alloc_point).call(block_identifier=block_identifier)

    def gov_token(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.govToken().call(block_identifier=block_identifier)

    def halving_update(self, cred:Credentials, _new_halving:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.halvingUpdate(_new_halving)
        return self.send_transaction(tx, cred)

    def liquidityaddr(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.liquidityaddr().call(block_identifier=block_identifier)

    def lock_update(self, cred:Credentials, _newlock:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.lockUpdate(_newlock)
        return self.send_transaction(tx, cred)

    def lockcom_update(self, cred:Credentials, _newcomlock:uint256) -> TxReceipt:
        tx = self.contract.functions.lockcomUpdate(_newcomlock)
        return self.send_transaction(tx, cred)

    def lockdev_update(self, cred:Credentials, _newdevlock:uint256) -> TxReceipt:
        tx = self.contract.functions.lockdevUpdate(_newdevlock)
        return self.send_transaction(tx, cred)

    def lockfounder_update(self, cred:Credentials, _newfounderlock:uint256) -> TxReceipt:
        tx = self.contract.functions.lockfounderUpdate(_newfounderlock)
        return self.send_transaction(tx, cred)

    def locklp_update(self, cred:Credentials, _newlplock:uint256) -> TxReceipt:
        tx = self.contract.functions.locklpUpdate(_newlplock)
        return self.send_transaction(tx, cred)

    def lp_update(self, cred:Credentials, _new_lp:address) -> TxReceipt:
        tx = self.contract.functions.lpUpdate(_new_lp)
        return self.send_transaction(tx, cred)

    def mass_update_pools(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.massUpdatePools()
        return self.send_transaction(tx, cred)

    def owner(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.owner().call(block_identifier=block_identifier)

    def pending_reward(self, _pid:uint256, _user:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.pendingReward(_pid, _user).call(block_identifier=block_identifier)

    def pool_existence(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.poolExistence(a).call(block_identifier=block_identifier)

    def pool_id1(self, a:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.poolId1(a).call(block_identifier=block_identifier)

    def pool_info(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[address, uint256, uint256, uint256]:
        return self.contract.functions.poolInfo(a).call(block_identifier=block_identifier)

    def pool_length(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.poolLength().call(block_identifier=block_identifier)

    def reclaim_token_ownership(self, cred:Credentials, _new_owner:address) -> TxReceipt:
        tx = self.contract.functions.reclaimTokenOwnership(_new_owner)
        return self.send_transaction(tx, cred)

    def remove_authorized(self, cred:Credentials, _to_remove:address) -> TxReceipt:
        tx = self.contract.functions.removeAuthorized(_to_remove)
        return self.send_transaction(tx, cred)

    def renounce_ownership(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.renounceOwnership()
        return self.send_transaction(tx, cred)

    def reward_mul_update(self, cred:Credentials, _new_mul_reward:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.rewardMulUpdate(_new_mul_reward)
        return self.send_transaction(tx, cred)

    def reward_update(self, cred:Credentials, _new_reward:uint256) -> TxReceipt:
        tx = self.contract.functions.rewardUpdate(_new_reward)
        return self.send_transaction(tx, cred)

    def set(self, cred:Credentials, _pid:uint256, _alloc_point:uint256, _with_update:bool) -> TxReceipt:
        tx = self.contract.functions.set(_pid, _alloc_point, _with_update)
        return self.send_transaction(tx, cred)

    def set_dev_fee_stage(self, cred:Credentials, _dev_fees:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setDevFeeStage(_dev_fees)
        return self.send_transaction(tx, cred)

    def set_stage_ends(self, cred:Credentials, _time_ends:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setStageEnds(_time_ends)
        return self.send_transaction(tx, cred)

    def set_stage_starts(self, cred:Credentials, _time_starts:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setStageStarts(_time_starts)
        return self.send_transaction(tx, cred)

    def set_user_fee_stage(self, cred:Credentials, _user_fees:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setUserFeeStage(_user_fees)
        return self.send_transaction(tx, cred)

    def start_timestamp_update(self, cred:Credentials, _newstarttimestamp:uint256) -> TxReceipt:
        tx = self.contract.functions.startTimestampUpdate(_newstarttimestamp)
        return self.send_transaction(tx, cred)

    def time_delta_end_stage(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.timeDeltaEndStage(a).call(block_identifier=block_identifier)

    def time_delta_start_stage(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.timeDeltaStartStage(a).call(block_identifier=block_identifier)

    def total_alloc_point(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.totalAllocPoint().call(block_identifier=block_identifier)

    def transfer_ownership(self, cred:Credentials, new_owner:address) -> TxReceipt:
        tx = self.contract.functions.transferOwnership(new_owner)
        return self.send_transaction(tx, cred)

    def update_pool(self, cred:Credentials, _pid:uint256) -> TxReceipt:
        tx = self.contract.functions.updatePool(_pid)
        return self.send_transaction(tx, cred)

    def usd_oracle(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.usdOracle().call(block_identifier=block_identifier)

    def user_delta(self, _pid:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.userDelta(_pid).call(block_identifier=block_identifier)

    def user_fee_stage(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.userFeeStage(a).call(block_identifier=block_identifier)

    def user_info(self, a:uint256, b:address, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint256, uint256, uint256, uint256, uint256]:
        return self.contract.functions.userInfo(a, b).call(block_identifier=block_identifier)

    def withdraw(self, cred:Credentials, _pid:uint256, _amount:uint256) -> TxReceipt:
        tx = self.contract.functions.withdraw(_pid, _amount)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xc20a268bc7c4dB28f1f6e1703676513Db06C1B93",
    "sd": "0xcd26DfD7EdAe42eD525266D9a53b466db4Ed4f7b"
}

ABI = """[
    {"name": "Canceled", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Deactivated", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Expired", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "HeroAssigned", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "HeroRemoved", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "IncreasedTier", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "userData", "type": "tuple", "indexed": false, "internalType": "struct PowerUpUserData", "components": [{"name": "isActivated", "type": "bool", "internalType": "bool"}, {"name": "emergencyWithdrawHappened", "type": "bool", "internalType": "bool"}, {"name": "tier", "type": "uint256", "internalType": "uint256"}, {"name": "openHeroSlots", "type": "uint256", "internalType": "uint256"}, {"name": "cancellationHeldSlots", "type": "uint256", "internalType": "uint256"}, {"name": "heldSlotExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}]}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "PowerUpAdded", "type": "event", "inputs": [{"name": "powerup", "type": "tuple", "indexed": false, "internalType": "struct PowerUp", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "powerUpType", "type": "uint256", "internalType": "uint256"}, {"name": "tiers", "type": "uint256", "internalType": "uint256"}, {"name": "heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "cancelDelay", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenPerTier", "type": "uint256[]", "internalType": "uint256[]"}]}], "anonymous": false},
    {"name": "PowerUpRemoved", "type": "event", "inputs": [{"name": "powerup", "type": "tuple", "indexed": false, "internalType": "struct PowerUp", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "powerUpType", "type": "uint256", "internalType": "uint256"}, {"name": "tiers", "type": "uint256", "internalType": "uint256"}, {"name": "heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "cancelDelay", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenPerTier", "type": "uint256[]", "internalType": "uint256[]"}]}], "anonymous": false},
    {"name": "PowerUpUpdated", "type": "event", "inputs": [{"name": "powerup", "type": "tuple", "indexed": false, "internalType": "struct PowerUp", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "powerUpType", "type": "uint256", "internalType": "uint256"}, {"name": "tiers", "type": "uint256", "internalType": "uint256"}, {"name": "heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "cancelDelay", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenPerTier", "type": "uint256[]", "internalType": "uint256[]"}]}], "anonymous": false},
    {"name": "RoleAdminChanged", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "previousAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "newAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}], "anonymous": false},
    {"name": "RoleGranted", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "RoleRevoked", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "Subscribed", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": false, "internalType": "address"}, {"name": "powerUpId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "userData", "type": "tuple", "indexed": false, "internalType": "struct PowerUpUserData", "components": [{"name": "isActivated", "type": "bool", "internalType": "bool"}, {"name": "emergencyWithdrawHappened", "type": "bool", "internalType": "bool"}, {"name": "tier", "type": "uint256", "internalType": "uint256"}, {"name": "openHeroSlots", "type": "uint256", "internalType": "uint256"}, {"name": "cancellationHeldSlots", "type": "uint256", "internalType": "uint256"}, {"name": "heldSlotExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}]}], "anonymous": false},
    {"name": "CANCELER_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "DEFAULT_ADMIN_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "MODERATOR_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "activePowerUps", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "addPowerUp", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}, {"name": "_name", "type": "string", "internalType": "string"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_tiers", "type": "uint256", "internalType": "uint256"}, {"name": "_heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "_lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "_cancelHoldDelay", "type": "uint256", "internalType": "uint256"}, {"name": "_govTokenPerTier", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "assignHero", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "assignHeroes", "type": "function", "inputs": [{"name": "_powerUpIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "cancel", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "cleanUpHeroAssignments", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "deactivatePowerUps", "type": "function", "inputs": [{"name": "_user", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "emergencyCancelAll", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getActivePowerUps", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "powerUpType", "type": "uint256", "internalType": "uint256"}, {"name": "tiers", "type": "uint256", "internalType": "uint256"}, {"name": "heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "cancelDelay", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenPerTier", "type": "uint256[]", "internalType": "uint256[]"}], "internalType": "struct PowerUp[]"}], "stateMutability": "view"},
    {"name": "getAssignedHeroIds", "type": "function", "inputs": [{"name": "_account", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "getAssignedHeroes", "type": "function", "inputs": [{"name": "_account", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum HeroStatus"}], "internalType": "struct HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroProfessions"}], "internalType": "struct Hero[]"}], "stateMutability": "view"},
    {"name": "getPowerUpHeroData", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "assignedSlot", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}], "internalType": "struct PowerUpHeroData"}], "stateMutability": "view"},
    {"name": "getRoleAdmin", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "getUserPowerUpData", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "isActivated", "type": "bool", "internalType": "bool"}, {"name": "emergencyWithdrawHappened", "type": "bool", "internalType": "bool"}, {"name": "tier", "type": "uint256", "internalType": "uint256"}, {"name": "openHeroSlots", "type": "uint256", "internalType": "uint256"}, {"name": "cancellationHeldSlots", "type": "uint256", "internalType": "uint256"}, {"name": "heldSlotExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}], "internalType": "struct PowerUpUserData"}], "stateMutability": "view"},
    {"name": "getUserPowerUpDataForActivePowerUps", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "isActivated", "type": "bool", "internalType": "bool"}, {"name": "emergencyWithdrawHappened", "type": "bool", "internalType": "bool"}, {"name": "tier", "type": "uint256", "internalType": "uint256"}, {"name": "openHeroSlots", "type": "uint256", "internalType": "uint256"}, {"name": "cancellationHeldSlots", "type": "uint256", "internalType": "uint256"}, {"name": "heldSlotExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}], "internalType": "struct PowerUpUserData[]"}, {"name": "", "type": "tuple[]", "components": [{"name": "powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "govTokens", "type": "uint256", "internalType": "uint256"}, {"name": "end", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "usedBalance", "type": "uint256", "internalType": "uint256"}], "internalType": "struct PowerUpLock[]"}], "stateMutability": "view"},
    {"name": "grantRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "hasRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "increaseTier", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_tier", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_voteEscrowRewardPoolAddress", "type": "address", "internalType": "address"}, {"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_questCoreAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "isHeroPowerUpActive", "type": "function", "inputs": [{"name": "_user", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "isUserPowerUpActive", "type": "function", "inputs": [{"name": "_user", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "powerUpHeroData", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "internalType": "uint256"}, {"name": "assignedSlot", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "powerUpHeroesAssigned", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "powerUps", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "powerUpType", "type": "uint256", "internalType": "uint256"}, {"name": "tiers", "type": "uint256", "internalType": "uint256"}, {"name": "heroesPerTier", "type": "uint256", "internalType": "uint256"}, {"name": "lockTimeRequiredToAcquire", "type": "uint256", "internalType": "uint256"}, {"name": "cancelDelay", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "removeHero", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "removeHeroFromAllType3", "type": "function", "inputs": [{"name": "_user", "type": "address", "internalType": "address"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "removeHeroFromAllType3ForSender", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "removeHeroes", "type": "function", "inputs": [{"name": "_powerUpIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "renounceRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "revokeRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "subscribe", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}, {"name": "_tier", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "supportsInterface", "type": "function", "inputs": [{"name": "interfaceId", "type": "bytes4", "internalType": "bytes4"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "userPowerUpData", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "isActivated", "type": "bool", "internalType": "bool"}, {"name": "emergencyWithdrawHappened", "type": "bool", "internalType": "bool"}, {"name": "tier", "type": "uint256", "internalType": "uint256"}, {"name": "openHeroSlots", "type": "uint256", "internalType": "uint256"}, {"name": "cancellationHeldSlots", "type": "uint256", "internalType": "uint256"}, {"name": "heldSlotExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "govTokenHoldExpiration", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "verifyAllPowerUps", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "verifyPowerUp", "type": "function", "inputs": [{"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "verifyPowerUpForUser", "type": "function", "inputs": [{"name": "_user", "type": "address", "internalType": "address"}, {"name": "_powerUpId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class PowerUpManager(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def active_power_ups(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.activePowerUps(a).call(block_identifier=block_identifier)

    def add_power_up(self, cred:Credentials, _id:uint256, _name:string, _type:uint256, _tiers:uint256, _heroes_per_tier:uint256, _lock_time_required_to_acquire:uint256, _cancel_hold_delay:uint256, _gov_token_per_tier:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.addPowerUp(_id, _name, _type, _tiers, _heroes_per_tier, _lock_time_required_to_acquire, _cancel_hold_delay, _gov_token_per_tier)
        return self.send_transaction(tx, cred)

    def assign_hero(self, cred:Credentials, _power_up_id:uint256, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.assignHero(_power_up_id, _hero_id)
        return self.send_transaction(tx, cred)

    def assign_heroes(self, cred:Credentials, _power_up_ids:Sequence[uint256], _hero_ids:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.assignHeroes(_power_up_ids, _hero_ids)
        return self.send_transaction(tx, cred)

    def cancel(self, cred:Credentials, _power_up_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancel(_power_up_id)
        return self.send_transaction(tx, cred)

    def clean_up_hero_assignments(self, cred:Credentials, _power_up_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cleanUpHeroAssignments(_power_up_id)
        return self.send_transaction(tx, cred)

    def deactivate_power_ups(self, cred:Credentials, _user:address) -> TxReceipt:
        tx = self.contract.functions.deactivatePowerUps(_user)
        return self.send_transaction(tx, cred)

    def emergency_cancel_all(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.emergencyCancelAll(_address)
        return self.send_transaction(tx, cred)

    def get_active_power_ups(self, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getActivePowerUps().call(block_identifier=block_identifier)

    def get_assigned_hero_ids(self, _account:address, _power_up_id:uint256, block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getAssignedHeroIds(_account, _power_up_id).call(block_identifier=block_identifier)

    def get_assigned_heroes(self, _account:address, _power_up_id:uint256, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getAssignedHeroes(_account, _power_up_id).call(block_identifier=block_identifier)

    def get_power_up_hero_data(self, _power_up_id:uint256, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getPowerUpHeroData(_power_up_id, _hero_id).call(block_identifier=block_identifier)

    def get_user_power_up_data(self, _address:address, _power_up_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getUserPowerUpData(_address, _power_up_id).call(block_identifier=block_identifier)

    def get_user_power_up_data_for_active_power_ups(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> Tuple[List[tuple], List[tuple]]:
        return self.contract.functions.getUserPowerUpDataForActivePowerUps(_address).call(block_identifier=block_identifier)

    def increase_tier(self, cred:Credentials, _power_up_id:uint256, _tier:uint256) -> TxReceipt:
        tx = self.contract.functions.increaseTier(_power_up_id, _tier)
        return self.send_transaction(tx, cred)

    def initialize(self, cred:Credentials, _vote_escrow_reward_pool_address:address, _hero_core_address:address, _quest_core_address:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_vote_escrow_reward_pool_address, _hero_core_address, _quest_core_address)
        return self.send_transaction(tx, cred)

    def is_hero_power_up_active(self, _user:address, _power_up_id:uint256, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.isHeroPowerUpActive(_user, _power_up_id, _hero_id).call(block_identifier=block_identifier)

    def is_user_power_up_active(self, _user:address, _power_up_id:uint256, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.isUserPowerUpActive(_user, _power_up_id).call(block_identifier=block_identifier)

    def power_up_hero_data(self, a:uint256, b:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint256, address]:
        return self.contract.functions.powerUpHeroData(a, b).call(block_identifier=block_identifier)

    def power_up_heroes_assigned(self, a:address, b:uint256, c:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.powerUpHeroesAssigned(a, b, c).call(block_identifier=block_identifier)

    def power_ups(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, string, uint256, uint256, uint256, uint256, uint256]:
        return self.contract.functions.powerUps(a).call(block_identifier=block_identifier)

    def remove_hero(self, cred:Credentials, _power_up_id:uint256, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.removeHero(_power_up_id, _hero_id)
        return self.send_transaction(tx, cred)

    def remove_hero_from_all_type3(self, cred:Credentials, _user:address, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.removeHeroFromAllType3(_user, _hero_id)
        return self.send_transaction(tx, cred)

    def remove_hero_from_all_type3_for_sender(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.removeHeroFromAllType3ForSender(_hero_id)
        return self.send_transaction(tx, cred)

    def remove_heroes(self, cred:Credentials, _power_up_ids:Sequence[uint256], _hero_ids:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.removeHeroes(_power_up_ids, _hero_ids)
        return self.send_transaction(tx, cred)

    def subscribe(self, cred:Credentials, _power_up_id:uint256, _tier:uint256) -> TxReceipt:
        tx = self.contract.functions.subscribe(_power_up_id, _tier)
        return self.send_transaction(tx, cred)

    def supports_interface(self, interface_id:bytes4, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.supportsInterface(interface_id).call(block_identifier=block_identifier)

    def user_power_up_data(self, a:address, b:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[bool, bool, uint256, uint256, uint256, uint256, uint256, address]:
        return self.contract.functions.userPowerUpData(a, b).call(block_identifier=block_identifier)

    def verify_all_power_ups(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.verifyAllPowerUps()
        return self.send_transaction(tx, cred)

    def verify_power_up(self, cred:Credentials, _power_up_id:uint256) -> TxReceipt:
        tx = self.contract.functions.verifyPowerUp(_power_up_id)
        return self.send_transaction(tx, cred)

    def verify_power_up_for_user(self, cred:Credentials, _user:address, _power_up_id:uint256) -> TxReceipt:
        tx = self.contract.functions.verifyPowerUpForUser(_user, _power_up_id)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xeaF833A0Ae97897f6F69a728C9c17916296cecCA",
    "sd": "0xe5D563F7e4144955FCFa8b90da45825426a05bD4"
}

ABI = """[
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "PetExchangeCompleted", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "eggId1", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "eggId2", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "eggTypeRecieved", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "PetExchangeStarted", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "eggId1", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "eggId2", "type": "uint256", "indexed": true, "internalType": "uint256"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "completeExchange", "type": "function", "inputs": [{"name": "_exchangeId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "eggs", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "getPetExchange", "type": "function", "inputs": [{"name": "_exchangeId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}, {"name": "petId1", "type": "uint256", "internalType": "uint256"}, {"name": "petId2", "type": "uint256", "internalType": "uint256"}, {"name": "seedblock", "type": "uint256", "internalType": "uint256"}, {"name": "finishTime", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "enum PetExchangeStatus"}], "internalType": "struct PetExchangeData"}], "stateMutability": "view"},
    {"name": "getUserPetExchanges", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}, {"name": "petId1", "type": "uint256", "internalType": "uint256"}, {"name": "petId2", "type": "uint256", "internalType": "uint256"}, {"name": "seedblock", "type": "uint256", "internalType": "uint256"}, {"name": "finishTime", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "enum PetExchangeStatus"}], "internalType": "struct PetExchangeData[]"}], "stateMutability": "view"},
    {"name": "idToPetExchange", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}, {"name": "petId1", "type": "uint256", "internalType": "uint256"}, {"name": "petId2", "type": "uint256", "internalType": "uint256"}, {"name": "seedblock", "type": "uint256", "internalType": "uint256"}, {"name": "finishTime", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "enum PetExchangeStatus"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_petCoreAddress", "type": "address", "internalType": "address"}, {"name": "_pastureAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "profileExchangedPets", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "owner", "type": "address", "internalType": "address"}, {"name": "petId1", "type": "uint256", "internalType": "uint256"}, {"name": "petId2", "type": "uint256", "internalType": "uint256"}, {"name": "seedblock", "type": "uint256", "internalType": "uint256"}, {"name": "finishTime", "type": "uint256", "internalType": "uint256"}, {"name": "status", "type": "uint8", "internalType": "enum PetExchangeStatus"}], "stateMutability": "view"},
    {"name": "setEgg", "type": "function", "inputs": [{"name": "_index", "type": "uint8", "internalType": "uint8"}, {"name": "_eggAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setPasture", "type": "function", "inputs": [{"name": "_pastureAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setPetCore", "type": "function", "inputs": [{"name": "_petCoreAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startExchange", "type": "function", "inputs": [{"name": "_petId1", "type": "uint256", "internalType": "uint256"}, {"name": "_petId2", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "totalExchanges", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class PetExchange(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def complete_exchange(self, cred:Credentials, _exchange_id:uint256) -> TxReceipt:
        tx = self.contract.functions.completeExchange(_exchange_id)
        return self.send_transaction(tx, cred)

    def eggs(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.eggs(a).call(block_identifier=block_identifier)

    def get_pet_exchange(self, _exchange_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getPetExchange(_exchange_id).call(block_identifier=block_identifier)

    def get_user_pet_exchanges(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getUserPetExchanges(_address).call(block_identifier=block_identifier)

    def id_to_pet_exchange(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint256, uint256, uint256, uint256, uint8]:
        return self.contract.functions.idToPetExchange(a).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _pet_core_address:address, _pasture_address:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_pet_core_address, _pasture_address)
        return self.send_transaction(tx, cred)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def profile_exchanged_pets(self, a:address, b:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint256, uint256, uint256, uint256, uint8]:
        return self.contract.functions.profileExchangedPets(a, b).call(block_identifier=block_identifier)

    def set_egg(self, cred:Credentials, _index:uint8, _egg_address:address) -> TxReceipt:
        tx = self.contract.functions.setEgg(_index, _egg_address)
        return self.send_transaction(tx, cred)

    def set_pasture(self, cred:Credentials, _pasture_address:address) -> TxReceipt:
        tx = self.contract.functions.setPasture(_pasture_address)
        return self.send_transaction(tx, cred)

    def set_pet_core(self, cred:Credentials, _pet_core_address:address) -> TxReceipt:
        tx = self.contract.functions.setPetCore(_pet_core_address)
        return self.send_transaction(tx, cred)

    def start_exchange(self, cred:Credentials, _pet_id1:uint256, _pet_id2:uint256) -> TxReceipt:
        tx = self.contract.functions.startExchange(_pet_id1, _pet_id2)
        return self.send_transaction(tx, cred)

    def total_exchanges(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.totalExchanges().call(block_identifier=block_identifier)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xE9AbfBC143d7cef74b5b793ec5907fa62ca53154",
    "sd": "0x8dc58d6327E1f65b18B82EDFb01A361f3AAEf624"
}

ABI = """[
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "QuestCanceled", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "quest", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "indexed": false, "internalType": "struct Quest"}], "anonymous": false},
    {"name": "QuestCompleted", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "quest", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "indexed": false, "internalType": "struct Quest"}], "anonymous": false},
    {"name": "QuestReward", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "rewardItem", "type": "address", "indexed": false, "internalType": "address"}, {"name": "itemQuantity", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "QuestSkillUp", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "profession", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "skillUp", "type": "uint16", "indexed": false, "internalType": "uint16"}], "anonymous": false},
    {"name": "QuestStaminaSpent", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "staminaFullAt", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "staminaSpent", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "QuestStarted", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "quest", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "indexed": false, "internalType": "struct Quest"}, {"name": "startAtTime", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "QuestXP", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "xpEarned", "type": "uint64", "indexed": false, "internalType": "uint64"}], "anonymous": false},
    {"name": "RewardMinted", "type": "event", "inputs": [{"name": "questId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "reward", "type": "address", "indexed": true, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "data", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "TrainingAttemptDone", "type": "event", "inputs": [{"name": "success", "type": "bool", "indexed": false, "internalType": "bool"}, {"name": "attempt", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}], "anonymous": false},
    {"name": "TrainingSuccessRatio", "type": "event", "inputs": [{"name": "winCount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "attempts", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": true, "internalType": "uint256"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "activateQuest", "type": "function", "inputs": [{"name": "_questAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "activeAccountQuests", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "stateMutability": "view"},
    {"name": "cancelQuest", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "clearActiveQuests", "type": "function", "inputs": [{"name": "_questAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "clearActiveQuestsAndHeroes", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "clearActiveQuestsAndHeroesWithOffset", "type": "function", "inputs": [{"name": "_offset", "type": "uint256", "internalType": "uint256"}, {"name": "_amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "completeQuest", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getAccountActiveQuests", "type": "function", "inputs": [{"name": "_account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "internalType": "struct Quest[]"}], "stateMutability": "view"},
    {"name": "getCurrentStamina", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getHeroQuest", "type": "function", "inputs": [{"name": "heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "internalType": "struct Quest"}], "stateMutability": "view"},
    {"name": "getQuestContracts", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address[]", "internalType": "address[]"}], "stateMutability": "view"},
    {"name": "heroCore", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IHeroCoreUpgradeable"}], "stateMutability": "view"},
    {"name": "heroToQuest", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "isQuest", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "multiCompleteQuest", "type": "function", "inputs": [{"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "multiStartQuest", "type": "function", "inputs": [{"name": "_questAddress", "type": "address[]", "internalType": "address[]"}, {"name": "_heroIds", "type": "uint256[][]", "internalType": "uint256[][]"}, {"name": "_attempts", "type": "uint8[]", "internalType": "uint8[]"}, {"name": "_level", "type": "uint8[]", "internalType": "uint8[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "questCounter", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "questRewarder", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "quests", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "questAddress", "type": "address", "internalType": "address"}, {"name": "level", "type": "uint8", "internalType": "uint8"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "startAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "completeAtTime", "type": "uint256", "internalType": "uint256"}, {"name": "attempts", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum QuestStatus"}], "stateMutability": "view"},
    {"name": "setQuestRewarder", "type": "function", "inputs": [{"name": "_questRewarder", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTimePerStamina", "type": "function", "inputs": [{"name": "_timePerStamina", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startQuest", "type": "function", "inputs": [{"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_questAddress", "type": "address", "internalType": "address"}, {"name": "_attempts", "type": "uint8", "internalType": "uint8"}, {"name": "_level", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "timePerStamina", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"}
]
"""     

class QuestCore(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def activate_quest(self, cred:Credentials, _quest_address:address) -> TxReceipt:
        tx = self.contract.functions.activateQuest(_quest_address)
        return self.send_transaction(tx, cred)

    def active_account_quests(self, a:address, b:address, c:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint8, address, uint256, uint256, uint256, uint8, uint8]:
        return self.contract.functions.activeAccountQuests(a, b, c).call(block_identifier=block_identifier)

    def cancel_quest(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancelQuest(_hero_id)
        return self.send_transaction(tx, cred)

    def clear_active_quests(self, cred:Credentials, _quest_address:address) -> TxReceipt:
        tx = self.contract.functions.clearActiveQuests(_quest_address)
        return self.send_transaction(tx, cred)

    def clear_active_quests_and_heroes(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.clearActiveQuestsAndHeroes()
        return self.send_transaction(tx, cred)

    def clear_active_quests_and_heroes_with_offset(self, cred:Credentials, _offset:uint256, _amount:uint256) -> TxReceipt:
        tx = self.contract.functions.clearActiveQuestsAndHeroesWithOffset(_offset, _amount)
        return self.send_transaction(tx, cred)

    def complete_quest(self, cred:Credentials, _hero_id:uint256) -> TxReceipt:
        tx = self.contract.functions.completeQuest(_hero_id)
        return self.send_transaction(tx, cred)

    def get_account_active_quests(self, _account:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getAccountActiveQuests(_account).call(block_identifier=block_identifier)

    def get_current_stamina(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getCurrentStamina(_hero_id).call(block_identifier=block_identifier)

    def get_hero_quest(self, hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getHeroQuest(hero_id).call(block_identifier=block_identifier)

    def get_quest_contracts(self, block_identifier:BlockIdentifier = 'latest') -> List[address]:
        return self.contract.functions.getQuestContracts().call(block_identifier=block_identifier)

    def hero_core(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.heroCore().call(block_identifier=block_identifier)

    def hero_to_quest(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.heroToQuest(a).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _hero_core_address:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_hero_core_address)
        return self.send_transaction(tx, cred)

    def is_quest(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.isQuest(a).call(block_identifier=block_identifier)

    def multi_complete_quest(self, cred:Credentials, _hero_ids:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.multiCompleteQuest(_hero_ids)
        return self.send_transaction(tx, cred)

    def multi_start_quest(self, cred:Credentials, _quest_address:Sequence[address], _hero_ids:Sequence[Sequence[uint256]], _attempts:Sequence[uint8], _level:Sequence[uint8]) -> TxReceipt:
        tx = self.contract.functions.multiStartQuest(_quest_address, _hero_ids, _attempts, _level)
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def quest_counter(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.questCounter().call(block_identifier=block_identifier)

    def quest_rewarder(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.questRewarder().call(block_identifier=block_identifier)

    def quests(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint8, address, uint256, uint256, uint256, uint8, uint8]:
        return self.contract.functions.quests(a).call(block_identifier=block_identifier)

    def set_quest_rewarder(self, cred:Credentials, _quest_rewarder:address) -> TxReceipt:
        tx = self.contract.functions.setQuestRewarder(_quest_rewarder)
        return self.send_transaction(tx, cred)

    def set_time_per_stamina(self, cred:Credentials, _time_per_stamina:uint256) -> TxReceipt:
        tx = self.contract.functions.setTimePerStamina(_time_per_stamina)
        return self.send_transaction(tx, cred)

    def start_quest(self, cred:Credentials, _hero_ids:Sequence[uint256], _quest_address:address, _attempts:uint8, _level:uint8) -> TxReceipt:
        tx = self.contract.functions.startQuest(_hero_ids, _quest_address, _attempts, _level)
        return self.send_transaction(tx, cred)

    def time_per_stamina(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.timePerStamina().call(block_identifier=block_identifier)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x6391F796D56201D279a42fD3141aDa7e26A3B4A5",
    "sd": "0xe1b8C354BE50357c2ab90A962254526d08aF0D2D"
}

ABI = """[
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "ProfileCreated", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": false, "internalType": "address"}, {"name": "name", "type": "string", "indexed": false, "internalType": "string"}, {"name": "created", "type": "uint64", "indexed": false, "internalType": "uint64"}, {"name": "nftId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "collectionId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "ProfileUpdated", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": false, "internalType": "address"}, {"name": "name", "type": "string", "indexed": false, "internalType": "string"}, {"name": "nftId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "collectionId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "MAX_CHAR", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "MAX_PIC", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "MIN_CHAR", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "addressToProfile", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "created", "type": "uint64", "internalType": "uint64"}, {"name": "nftId", "type": "uint256", "internalType": "uint256"}, {"name": "collectionId", "type": "uint256", "internalType": "uint256"}, {"name": "picUri", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "adminCreateProfile", "type": "function", "inputs": [{"name": "_owner", "type": "address", "internalType": "address"}, {"name": "_name", "type": "string", "internalType": "string"}, {"name": "_created", "type": "uint64", "internalType": "uint64"}, {"name": "_nftId", "type": "uint256", "internalType": "uint256"}, {"name": "_collectionId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "batchSetPicURI", "type": "function", "inputs": [{"name": "_uriArray", "type": "string[]", "internalType": "string[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "changeName", "type": "function", "inputs": [{"name": "_profileAddress", "type": "address", "internalType": "address"}, {"name": "_name", "type": "string", "internalType": "string"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "changePic", "type": "function", "inputs": [{"name": "_profileAddress", "type": "address", "internalType": "address"}, {"name": "_nftId", "type": "uint256", "internalType": "uint256"}, {"name": "_collectionId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "createProfile", "type": "function", "inputs": [{"name": "_name", "type": "string", "internalType": "string"}, {"name": "_nftId", "type": "uint256", "internalType": "uint256"}, {"name": "_collectionId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getProfile", "type": "function", "inputs": [{"name": "_profileAddress", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "created", "type": "uint64", "internalType": "uint64"}, {"name": "nftId", "type": "uint256", "internalType": "uint256"}, {"name": "collectionId", "type": "uint256", "internalType": "uint256"}, {"name": "picUri", "type": "string", "internalType": "string"}], "internalType": "struct ProfileTypes.Profile"}], "stateMutability": "view"},
    {"name": "getProfileByAddress", "type": "function", "inputs": [{"name": "_profileAddress", "type": "address", "internalType": "address"}], "outputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}, {"name": "_owner", "type": "address", "internalType": "address"}, {"name": "_name", "type": "string", "internalType": "string"}, {"name": "_created", "type": "uint64", "internalType": "uint64"}, {"name": "_picId", "type": "uint8", "internalType": "uint8"}, {"name": "_heroId", "type": "uint256", "internalType": "uint256"}, {"name": "_points", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getProfileByName", "type": "function", "inputs": [{"name": "_name", "type": "string", "internalType": "string"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "name", "type": "string", "internalType": "string"}, {"name": "created", "type": "uint64", "internalType": "uint64"}, {"name": "nftId", "type": "uint256", "internalType": "uint256"}, {"name": "collectionId", "type": "uint256", "internalType": "uint256"}, {"name": "picUri", "type": "string", "internalType": "string"}], "internalType": "struct ProfileTypes.Profile"}], "stateMutability": "view"},
    {"name": "getTokenUrisHeldByAddress", "type": "function", "inputs": [{"name": "_profileAddress", "type": "address", "internalType": "address"}, {"name": "_collectionId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "string[]", "internalType": "string[]"}], "stateMutability": "view"},
    {"name": "heroesNftContract", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IHeroCore"}], "stateMutability": "view"},
    {"name": "identityTokenRouter", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IIdentityTokenRouter"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_identityTokenRouter", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "maxChar", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "maxPic", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "minChar", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "nameToAddress", "type": "function", "inputs": [{"name": "", "type": "string", "internalType": "string"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "picUris", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "setHeroes", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setIdentityTokenRouter", "type": "function", "inputs": [{"name": "_identityTokenRouter", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setNameLengths", "type": "function", "inputs": [{"name": "_min", "type": "uint8", "internalType": "uint8"}, {"name": "_max", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setPicMax", "type": "function", "inputs": [{"name": "_max", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setPicURI", "type": "function", "inputs": [{"name": "_picId", "type": "uint256", "internalType": "uint256"}, {"name": "_picUri", "type": "string", "internalType": "string"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class Profiles(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def max_char(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.MAX_CHAR().call(block_identifier=block_identifier)

    def max_pic(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.MAX_PIC().call(block_identifier=block_identifier)

    def min_char(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.MIN_CHAR().call(block_identifier=block_identifier)

    def address_to_profile(self, a:address, block_identifier:BlockIdentifier = 'latest') -> Tuple[address, string, uint64, uint256, uint256, string]:
        return self.contract.functions.addressToProfile(a).call(block_identifier=block_identifier)

    def admin_create_profile(self, cred:Credentials, _owner:address, _name:string, _created:uint64, _nft_id:uint256, _collection_id:uint256) -> TxReceipt:
        tx = self.contract.functions.adminCreateProfile(_owner, _name, _created, _nft_id, _collection_id)
        return self.send_transaction(tx, cred)

    def batch_set_pic_uri(self, cred:Credentials, _uri_array:Sequence[string]) -> TxReceipt:
        tx = self.contract.functions.batchSetPicURI(_uri_array)
        return self.send_transaction(tx, cred)

    def change_name(self, cred:Credentials, _profile_address:address, _name:string) -> TxReceipt:
        tx = self.contract.functions.changeName(_profile_address, _name)
        return self.send_transaction(tx, cred)

    def change_pic(self, cred:Credentials, _profile_address:address, _nft_id:uint256, _collection_id:uint256) -> TxReceipt:
        tx = self.contract.functions.changePic(_profile_address, _nft_id, _collection_id)
        return self.send_transaction(tx, cred)

    def create_profile(self, cred:Credentials, _name:string, _nft_id:uint256, _collection_id:uint256) -> TxReceipt:
        tx = self.contract.functions.createProfile(_name, _nft_id, _collection_id)
        return self.send_transaction(tx, cred)

    def get_profile(self, _profile_address:address, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getProfile(_profile_address).call(block_identifier=block_identifier)

    def get_profile_by_address(self, _profile_address:address, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, string, uint64, uint8, uint256, uint256]:
        return self.contract.functions.getProfileByAddress(_profile_address).call(block_identifier=block_identifier)

    def get_profile_by_name(self, _name:string, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getProfileByName(_name).call(block_identifier=block_identifier)

    def get_token_uris_held_by_address(self, _profile_address:address, _collection_id:uint256, block_identifier:BlockIdentifier = 'latest') -> List[string]:
        return self.contract.functions.getTokenUrisHeldByAddress(_profile_address, _collection_id).call(block_identifier=block_identifier)

    def heroes_nft_contract(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.heroesNftContract().call(block_identifier=block_identifier)

    def identity_token_router(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.identityTokenRouter().call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _hero_core_address:address, _identity_token_router:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_hero_core_address, _identity_token_router)
        return self.send_transaction(tx, cred)

    def max_char(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.maxChar().call(block_identifier=block_identifier)

    def max_pic(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.maxPic().call(block_identifier=block_identifier)

    def min_char(self, block_identifier:BlockIdentifier = 'latest') -> uint8:
        return self.contract.functions.minChar().call(block_identifier=block_identifier)

    def name_to_address(self, a:string, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.nameToAddress(a).call(block_identifier=block_identifier)

    def pic_uris(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> string:
        return self.contract.functions.picUris(a).call(block_identifier=block_identifier)

    def set_heroes(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.setHeroes(_address)
        return self.send_transaction(tx, cred)

    def set_identity_token_router(self, cred:Credentials, _identity_token_router:address) -> TxReceipt:
        tx = self.contract.functions.setIdentityTokenRouter(_identity_token_router)
        return self.send_transaction(tx, cred)

    def set_name_lengths(self, cred:Credentials, _min:uint8, _max:uint8) -> TxReceipt:
        tx = self.contract.functions.setNameLengths(_min, _max)
        return self.send_transaction(tx, cred)

    def set_pic_max(self, cred:Credentials, _max:uint8) -> TxReceipt:
        tx = self.contract.functions.setPicMax(_max)
        return self.send_transaction(tx, cred)

    def set_pic_uri(self, cred:Credentials, _pic_id:uint256, _pic_uri:string) -> TxReceipt:
        tx = self.contract.functions.setPicURI(_pic_id, _pic_uri)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x77D991987ca85214f9686131C58c1ABE4C93E547",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "AuctionCancelled", "type": "event", "inputs": [{"name": "auctionId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "tokenId", "type": "uint256", "indexed": true, "internalType": "uint256"}], "anonymous": false},
    {"name": "AuctionCreated", "type": "event", "inputs": [{"name": "auctionId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "tokenId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "startingPrice", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "endingPrice", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "duration", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "winner", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "AuctionSuccessful", "type": "event", "inputs": [{"name": "auctionId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "tokenId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "totalPrice", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "winner", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "ERC721", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IERC721Upgradeable"}], "stateMutability": "view"},
    {"name": "auctionIdOffset", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "auctions", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "seller", "type": "address", "internalType": "address"}, {"name": "tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "startingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "endingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "duration", "type": "uint64", "internalType": "uint64"}, {"name": "startedAt", "type": "uint64", "internalType": "uint64"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "open", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "bid", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "_bidAmount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "bidFor", "type": "function", "inputs": [{"name": "_bidder", "type": "address", "internalType": "address"}, {"name": "_tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "_bidAmount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "cancelAuction", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "cancelAuctionWhenPaused", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "createAuction", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "_startingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "_endingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "_duration", "type": "uint64", "internalType": "uint64"}, {"name": "_winner", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getAuction", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "seller", "type": "address", "internalType": "address"}, {"name": "tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "startingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "endingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "duration", "type": "uint64", "internalType": "uint64"}, {"name": "startedAt", "type": "uint64", "internalType": "uint64"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "open", "type": "bool", "internalType": "bool"}], "internalType": "struct ERC721AuctionBase.Auction"}], "stateMutability": "view"},
    {"name": "getAuctions", "type": "function", "inputs": [{"name": "_tokenIds", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "seller", "type": "address", "internalType": "address"}, {"name": "tokenId", "type": "uint256", "internalType": "uint256"}, {"name": "startingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "endingPrice", "type": "uint128", "internalType": "uint128"}, {"name": "duration", "type": "uint64", "internalType": "uint64"}, {"name": "startedAt", "type": "uint64", "internalType": "uint64"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "open", "type": "bool", "internalType": "bool"}], "internalType": "struct ERC721AuctionBase.Auction[]"}], "stateMutability": "view"},
    {"name": "getCurrentPrice", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getUserAuctions", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_landCoreAddress", "type": "address", "internalType": "address"}, {"name": "_jewelTokenAddress", "type": "address", "internalType": "address"}, {"name": "_cut", "type": "uint256", "internalType": "uint256"}, {"name": "_auctionIdOffset", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "isOnAuction", "type": "function", "inputs": [{"name": "_tokenId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "jewelToken", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IJewelToken"}], "stateMutability": "view"},
    {"name": "maxPrice", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "minPrice", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "onERC721Received", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "bytes", "internalType": "bytes"}], "outputs": [{"name": "", "type": "bytes4", "internalType": "bytes4"}], "stateMutability": "pure"},
    {"name": "ownerCut", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "setFees", "type": "function", "inputs": [{"name": "_feeAddresses", "type": "address[]", "internalType": "address[]"}, {"name": "_feePercents", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setLimits", "type": "function", "inputs": [{"name": "_min", "type": "uint256", "internalType": "uint256"}, {"name": "_max", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "totalAuctions", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "userAuctions", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"}
]
"""     

class LandAuction(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def erc721(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.ERC721().call(block_identifier=block_identifier)

    def auction_id_offset(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.auctionIdOffset().call(block_identifier=block_identifier)

    def auctions(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[address, uint256, uint128, uint128, uint64, uint64, address, bool]:
        return self.contract.functions.auctions(a).call(block_identifier=block_identifier)

    def bid(self, cred:Credentials, _token_id:uint256, _bid_amount:uint256) -> TxReceipt:
        tx = self.contract.functions.bid(_token_id, _bid_amount)
        return self.send_transaction(tx, cred)

    def bid_for(self, cred:Credentials, _bidder:address, _token_id:uint256, _bid_amount:uint256) -> TxReceipt:
        tx = self.contract.functions.bidFor(_bidder, _token_id, _bid_amount)
        return self.send_transaction(tx, cred)

    def cancel_auction(self, cred:Credentials, _token_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancelAuction(_token_id)
        return self.send_transaction(tx, cred)

    def cancel_auction_when_paused(self, cred:Credentials, _token_id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancelAuctionWhenPaused(_token_id)
        return self.send_transaction(tx, cred)

    def create_auction(self, cred:Credentials, _token_id:uint256, _starting_price:uint128, _ending_price:uint128, _duration:uint64, _winner:address) -> TxReceipt:
        tx = self.contract.functions.createAuction(_token_id, _starting_price, _ending_price, _duration, _winner)
        return self.send_transaction(tx, cred)

    def get_auction(self, _token_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getAuction(_token_id).call(block_identifier=block_identifier)

    def get_auctions(self, _token_ids:Sequence[uint256], block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getAuctions(_token_ids).call(block_identifier=block_identifier)

    def get_current_price(self, _token_id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getCurrentPrice(_token_id).call(block_identifier=block_identifier)

    def get_user_auctions(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getUserAuctions(_address).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _land_core_address:address, _jewel_token_address:address, _cut:uint256, _auction_id_offset:uint256) -> TxReceipt:
        tx = self.contract.functions.initialize(_land_core_address, _jewel_token_address, _cut, _auction_id_offset)
        return self.send_transaction(tx, cred)

    def is_on_auction(self, _token_id:uint256, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.isOnAuction(_token_id).call(block_identifier=block_identifier)

    def jewel_token(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.jewelToken().call(block_identifier=block_identifier)

    def max_price(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.maxPrice().call(block_identifier=block_identifier)

    def min_price(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.minPrice().call(block_identifier=block_identifier)

    def on_erc721_received(self, a:address, b:address, c:uint256, d:bytes, block_identifier:BlockIdentifier = 'latest') -> bytes4:
        return self.contract.functions.onERC721Received(a, b, c, d).call(block_identifier=block_identifier)

    def owner_cut(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.ownerCut().call(block_identifier=block_identifier)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def set_fees(self, cred:Credentials, _fee_addresses:Sequence[address], _fee_percents:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setFees(_fee_addresses, _fee_percents)
        return self.send_transaction(tx, cred)

    def set_limits(self, cred:Credentials, _min:uint256, _max:uint256) -> TxReceipt:
        tx = self.contract.functions.setLimits(_min, _max)
        return self.send_transaction(tx, cred)

    def total_auctions(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.totalAuctions().call(block_identifier=block_identifier)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)

    def user_auctions(self, a:address, b:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.userAuctions(a, b).call(block_identifier=block_identifier)